<?php

namespace App\Exceptions;

use Exception;
use Illuminate\Http\Exception\HttpResponseException;
use Illuminate\Http\Response;
use Illuminate\Validation\ValidationException;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Symfony\Component\Debug\Exception\FatalThrowableError;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Laravel\Lumen\Exceptions\Handler as ExceptionHandler;
use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\UnsupportedMediaTypeHttpException;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that should not be reported.
     *
     * @var array
     */
    protected $dontReport = [
        AuthorizationException::class,
        HttpException::class,
        ModelNotFoundException::class,
        ValidationException::class,
    ];

    /**
     * Report or log an exception.
     *
     * This is a great spot to send exceptions to Sentry, Bugsnag, etc.
     *
     * @param  \Exception $e
     * @return void
     * @throws Exception
     */
    public function report(Exception $e)
    {
        parent::report($e);
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  \Exception $e
     * @return \Illuminate\Http\Response
     */
    public function render($request, Exception $e)
    {

        $response = [
            'success' => false,
            'status' => 400,
            'message' => ""
        ];

        if ($e instanceof ValidationException) {
            $response['status'] = Response::HTTP_BAD_REQUEST;
            $response['message'] = "Invalid HTTP Request object.";

        } elseif ($e instanceof NotFoundHttpException) {
            $response['status'] = Response::HTTP_NOT_FOUND;
            $response['message'] = "Resource not Found";

        }
        elseif ($e instanceof MethodNotAllowedHttpException) {
            $response['status'] = Response::HTTP_METHOD_NOT_ALLOWED;
            $response['message'] = " HTTP Method not allowed";

        } elseif ($e instanceof AuthorizationException) {
            $response['status'] = Response::HTTP_FORBIDDEN;

        } elseif ($e instanceof ValidationException) {
            $response['status'] = Response::HTTP_BAD_REQUEST;
            $response['message'] = "Bad Request sent";

        }  elseif ($e instanceof UnsupportedMediaTypeHttpException) {
            $response['status'] = Response::HTTP_UNSUPPORTED_MEDIA_TYPE;
            $response['message'] = "HTTP UNSUPPORTED MEDIA TYPE";

        }

        elseif ($e instanceof HttpResponseException) {
            $response['status']   = Response::HTTP_INTERNAL_SERVER_ERROR;
            $response['message'] = "Internal Server Error. Please contact the Server Administrator";

        }
        elseif ($e instanceof FatalThrowableError ) {
            $response['status']   = Response::HTTP_INTERNAL_SERVER_ERROR;
            $response['message'] = $e->getMessage();
        }

        return response()->json($response, $response['status']);

    }
}
