<?php

namespace App\Http\Controllers;

use App\Setting;
use Carbon\Carbon;
use Illuminate\Http\Request;

use App\Http\Requests;
use Illuminate\Support\Facades\Hash;
use Validator;
use App\Company;
use App\User;
use DB;
use App\Traits\SettingTrait;

class CustomerController extends Controller{

    use SettingTrait;

    public function __construct()
    {
        //
    }

    public function getCustomers(Request $request){
        try {
            if ($request->has('member_id')) {
                $customer = \App\User::where('id', $request->member_id)->select('firstname', 'lastname', 'email', 'company_id', 'phone', 'address')->get();
                return response()->json(['data' => $customer, 'status' => '200'], 200);
            }
            $customers = \App\User::select('firstname', 'lastname', 'email', 'company_id', 'phone', 'address', 'member_id')->get();
            return response()->json(['data' => $customers, 'status' => '200'], 200);
        }catch (\Exception $e){
            return response()->json(['data' => 'An Error Occurred '. $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function postCustomerStatus(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required',
            'status' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();


        if($userExists > 0) {
            $status = \App\User::where(
                [
                    ['id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0]]
                ]
            )->update(['status' => $request->status]);
            return response()->json(['data' => 'Member status changed', 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);

        }
    }

    public function MemberCreate(Request $request)
    {

        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;

        $reason= "Opening Balance";
        
        $status = 1;

        //Validate request
        $valid = Validator::make($request->all(), [
            'firstname' => 'bail|required|max:255',
            'lastname' => 'bail|required|max:255',
            'email' => 'bail|required|email|max:255',
            'member_id' => 'bail|required|unique:users,member_id',
            'password' => 'bail|required|min:6|confirmed',
            'client_number' => 'bail|required'
        ]);

        if ($valid->fails()) {
            return response()->json(['data' => $valid->errors()->first(), 'status' => '400'], 400);
        }

        //Get company ID
        $company = Company::where('client_number', $request->client_number)->select('id')->get();
        if($company->count() == 0){
            return response()->json(['data' => 'Company does not exist - invalid client_number', 'status' => '400'], 400);
        }

        $currency = $this->currency($company->first()->id);

        $userExists = User::where([
                ['company_id', '=', $company->first()->id],
                ['email', '=', $request->email]
            ]
        )->exists();

        if ($userExists) {
            return response()->json(['data' => 'Customer already registered to '.$company->name, 'status' => '400'], 400);
        }

        if($request->has('state_id')) {
            $location = DB::table('states')->where('state_id', '=', $request->state_id)->first();
        }

        $pin = $this->generatePIN();

        //Start transaction
        DB::beginTransaction();
        try {
            //Create a new member
            $member = new User();
            $member->firstname = $request['firstname'];
            $member->member_id = $request['member_id'];
            $member->lastname = $request['lastname'];
            $member->company_id = $company->first()->id;
            $member->name = $request['firstname'] . " " . $request['lastname'];
            $member->email = $request['email'];
            $member->phone = $request['phone'];
            $member->address = $request->has('state_id') ? $request['address'] : '';
            $member->country_name = $request->has('state_id') ? $location->country_name : null;
            $member->state_name = $request->has('state_id') ? $location->state_name : null;
            $member->city_name = $request->has('state_id') ? $location->state_id : null;
            $member->country_id = $request->has('state_id') ? $location->country_id : null;
            $member->state_id = $request->has('state_id')  ? $location->state_id : null;
            $member->city_id = $request->has('state_id') ? $location->city_id : null;
            $member->status = $status;
            $member->password = Hash::make($request['password']);
            $member->pin = Hash::make($pin);
            $member->save();

            //Credit the member just created
            $account = new \App\Accounts();
            $account->user_id = $member->id;
            $account->point = $request->has('point') ? $request['point'] : 0;
            $account->company_id = $company->first()->id;
            $account->save();
                
                if ($request['point'] > 0) {
                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['credit'];
                $transaction->type_name = array_keys($this->transtype)[1];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $request->point;
                $transaction->balance_raw = $request->point;
                $transaction->balance_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $company->first()->id;
                $transaction->member_id = $member->id;
                $transaction->point_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->reason = $reason ;
                $transaction->trans_by = json_encode($transby);
                $transaction->save();
            }
            
            //Check if there's any error, rollback transaction is any error
            if(!$member || !$account){
                DB::rollBack();
            }
            //Commit transaction
            DB::commit();

            $setting = DB::table('setting')->where('company_id', '=', $company->first()->id)->select('has_pin_validation')->first();
            $response = [
                'firstname' => $member->firstname,
                'lastname' => $member->lastname,
                'email' => $member->email,
                'client_number' => $request->client_number,
                'member_number' => $member->member_id,
                'member_id' => $member->id,
                'phone' => $member->phone,
                'status' => $member->status,
                'balance_raw' => $account->point,
                'pin' => $setting->has_pin_validation == 1 ? $pin : '',
                'balance_virtual' => $account->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
            ];

            //Return data with the newly created Member
            return response()->json(['data' => $response, 'status' => '200'], 200);

        }catch (\Exception $e) {
            DB::rollBack();
            return response()->json(['data' => "An Error Occurred" . $e->getMessage(), 'status' => '400'], 400);
        }

    }


    public function MemberUpdate(Request $request)
    {

        //Validate request
        $valid = Validator::make($request->all(), [
            'firstname' => 'bail|required|max:255',
            'lastname' => 'bail|required|max:255',
            'email' => 'bail|required|email|max:255',
            'member_id' => 'bail|required',
            'client_number' => 'bail|required'
        ]);


        if ($valid->fails()) {
            return response()->json(['data' => $valid->errors()->first(), 'status' => '400'], 400);
        }


        //Get company ID
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }



        try {
            //update member
            User::where('id', $request['member_id'])
            ->update(
                [
                    'firstname' => $request['firstname'],
                    'lastname' => $request['lastname'],
                    'name' => ($request['firstname'] . " " . $request['lastname']),
                    'phone' => $request['phone'],
                    'email' => $request['email'],
//                    'status' => $status,
//                $member->address = $request['address'];
//                $member->country_name = $request['country_name'];
//                $member->state_name = $request['state_name'];
//                $member->city_name = $request['city_name'];
//                $member->country_id = $request['country_id'];
//                $member->state_id = $request['state_id'];
//                $member->city_id = $request['city_id'];
                ]
            );

            return response()->json(['data' => ' Account updated', 'status' => '200'], 200);

        }catch (\PDOException $e) {
            DB::rollBack();
            return response()->json(['data' => "An Error Occurred. Could not update account", 'reason' => $e->getMessage(), 'status' => '400'], 400);
        }

    }

    public function customerTransactions(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $company = Company::where('client_number', $request->client_number)->select('id')->first();

        if($company->count() == 0){
            return response()->json(['data' => 'Company record not found', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $company->id]
        ])->exists();

        try{
            if($userExists) {
                $transaction =  \App\Transaction::leftJoin('users', 'users.id', '=', 'transactions.member_id')
                    ->where([
                        ['users.company_id', '=',  $company->id],
                        ['users.id', '=', $request->member_id]
                    ])
                    ->select(
                        'transactions.id as trans_id',
                        'transactions.type as trans_type',
                        'transactions.type_name',
                        'transactions.is_internal_action',
                        'transactions.is_internal_action_name',
                        'transactions.order_id',
                        'transactions.trans_by',
                        'transactions.point_raw',
                        'transactions.point_virtual',
                        'transactions.balance_raw',
                        'transactions.balance_virtual',
                        'users.email as email',
                        'users.firstname',
                        'users.lastname'
                    )
                    ->get ();

                if($transaction->isEmpty()){
                    return response()->json(['data' => 'No Transaction found for User', 'status' => '400'], 400);
                } else {
                return response()->json(['data' => $transaction, 'status' => '200'], 200);
                }
            }
            return response()->json(['data' => 'No records found for User', 'status' => '400'], 400);
        }catch(\PDOException $e){
            return response()->json(['data' => $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function customerCredit(Request $request){
        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;


        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required',
            'point' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');
        $currency = $this->currency($companyid[0]);
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();


//        $balance = \App\Accounts::where([
//            ['user_id', '=', $request->member_id],
//            ['company_id', '=', $companyid[0] ],
//        ])->select('point');

        try{
            if($userExists > 0){

                DB::beginTransaction();

                // Old balance
                $oldbalance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                //Update the member's account
                \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->increment('point', $request->point);

                $balance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['credit'];
                $transaction->type_name = array_keys($this->transtype)[1];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $request->point;
                $transaction->balance_raw = $balance->point;
                $transaction->balance_virtual = $balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $companyid[0];
                $transaction->member_id = $request->member_id;
                $transaction->point_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->reason = '';
                $transaction->trans_by = json_encode($transby);
                $transaction->save();


                //Check if a total record for the member exists
                $totalExists =  \App\MemberTotal::where(
                    [
                        ['member_id', $request->member_id],
                        ['company_id', $companyid[0]]
                    ]
                )->count();

                //Check if total for the member exists before updating the user's total credit
                if ($totalExists > 0){
                    //Update totals if a record exists
                    \App\MemberTotal::where(
                        [
                            ['member_id', $request->member_id],
                            ['company_id', $companyid[0]]
                        ]
                    )->increment('credit', $request->point);
                }else{
                    //Create a new record if it doesnt exist
                    \App\MemberTotal::firstOrCreate(
                        [
                            'member_id' => $request->member_id,
                            'company_id' => $companyid[0],
                            'credit' => $request->point
                        ]
                    );
                }

                DB::commit();

                //Get the member and and current point
                $customersAccount = \App\User::where('users.id', "=", $request->member_id)
                    ->select(
                        'users.firstname',
                        'users.lastname',
                        'users.email',
                        'users.company_id',
                        'users.id as member_id'
                    )
                    ->first();

                $response = [
                    'firstname' => $customersAccount->firstname,
                    'lastname' => $customersAccount->lastname,
                    'email' => $customersAccount->email,
                    'company_id' => $customersAccount->company_id,
                    'member_id' => $customersAccount->member_id,
                    'credit_raw' => $request->point,
                    'credit_virtual' => $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                    'member_number' => $customersAccount->member_id,
                    'balance_raw' => $transaction->balance_raw,
                    'old_balance_raw' => $oldbalance->point,
                    'balance_virtual' => $transaction->balance_virtual,
                    'old_balance_virtual' => $oldbalance->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                    'type_name' => $transaction->type_name
                ];

                return response()->json(['data' => $response, 'status' => '200'], 200);

            }else{
                return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);
            }

        }catch (\PDOException  $e){
            DB::rollBack();
            return response()->json(['data' => "Failed to Credit Member",'reason' => $e->getMessage(), 'status' => '400'], 400);
        }
    }

    //Debit a customer
    public function customerDebit(Request $request){
        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required',
            'point' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');
        $currency = $this->currency($companyid[0]);
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        try{
            if($userExists > 0){
                DB::beginTransaction();

                // Old balance
                $oldbalance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $account =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->decrement('point', $request->point);

                $balance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['debit'];
                $transaction->type_name = array_keys($this->transtype)[0];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $request->point;
                $transaction->point_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->balance_raw = $balance->point;
                $transaction->balance_virtual = $balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $companyid[0];
                $transaction->member_id = $request->member_id;
                $transaction->reason = '';
                $transaction->trans_by = json_encode($transby);
                $transaction->save();


                //Check if a total record for the member exists
                $totalExists =  \App\MemberTotal::where(
                    [
                        ['member_id', $request->member_id],
                        ['company_id', $companyid[0]]
                    ]
                )->count();

                //Check if total for the member exists before updating the user's total credit
                if ($totalExists > 0){
                    //Update totals if a record exists
                    \App\MemberTotal::where(
                        [
                            ['member_id', $request->member_id],
                            ['company_id', $companyid[0]]
                        ]
                    )->increment('credit', $request->point);
                }else{
                    //Create a new record if it doesnt exist
                    \App\MemberTotal::firstOrCreate(
                        [
                            'member_id' => $request->member_id,
                            'company_id' => $companyid[0],
                            'credit' => $request->point
                        ]
                    );
                }

                DB::commit();

                if($account){
                    $customersAccount = \App\User::where('users.id', "=", $request->member_id)
                        ->select(
                            'users.firstname',
                            'users.lastname',
                            'users.email',
                            'users.company_id',
                            'users.id as member_id',
                            'users.member_id as member_number'
                        )
                        ->first();

                    $response = [
                        'firstname' => $customersAccount->firstname,
                        'lastname' => $customersAccount->lastname,
                        'email' => $customersAccount->email,
                        'company_id' => $customersAccount->company_id,
                        'member_id' => $customersAccount->member_id,
                        'debit_raw' => $request->point,
                        'debit_virtual' => $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'member_number' => $customersAccount->member_id,
                        'balance_raw' => $transaction->balance_raw,
                        'balance_virtual' => $transaction->balance_virtual,
                        'balance_raw' => $transaction->balance_raw,
                        'old_balance_raw' => $oldbalance->point,
                        'balance_virtual' => $transaction->balance_virtual,
                        'old_balance_virtual' => $oldbalance->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'type_name' => $transaction->type_name,
                    ];
                    return response()->json(['data' => $response, 'status' => '200'], 200);
                }else{
                    return response()->json(['data' => 'Failed to update Account', 'status' => '200'], 200);
                }

            }else{
                return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);
            }

        }catch (\Exception $e){
            DB::rollBack();
            return response()->json(['data' => "Failed to Debit Member",'reason' => $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function getMemberPointTotalCredit(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $totalCredit = \App\MemberTotal::where([
                ['company_id', '=', $companyid[0] ],
                ['member_id', '=', $request->member_id ]
            ])->select('credit')->first();
            return response()->json(['data' => $totalCredit, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
    }

    public function getMemberPointTotalDebit(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status'=>'400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $totalCredit = \App\MemberTotal::where([
                ['company_id', '=', $companyid[0] ],
                ['member_id', '=', $request->member_id ]
            ])->select('debit')->first();
            return response()->json(['data' => $totalCredit, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'User does not exist', 'status' => '400'], 400);
        }
    }

    public function customerAccountTotal(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $currentBalance = \App\Accounts::where([
                ['company_id', '=', $companyid[0] ],
                ['user_id', '=', $request->member_id ]
            ])->select('point as current_balance')->first();
            return response()->json(['data' => $currentBalance, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'User does not exist', 'status' => '400'], 400);
        }

    }

    public function getCustomersPlusAccount(Request $request){
        $customersAccount = \App\User::join('accounts', function ($join){
            $join->on('users.id', '=', 'accounts.user_id');
            $join->on('accounts.company_id', '=', 'users.company_id');
        })
            ->select(
                'users.firstname',
                'users.lastname',
                'users.email',
                'users.company_id',
                'users.phone',
                'users.address',
                'accounts.point as account'
            )
            ->get();
        return response()->json(['data' => $customersAccount, 'status' => '200'], 200);
    }

    private function generatePIN(){
            //generate a random number between 0 and 9.
            return mt_rand(0000, 9999);
    }

    public function DebitCustomerPoints($client_no, $member_id, $points, $reason){
        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;


        $companyid = Company::where('client_number', $client_no)->pluck('id');
        $currency = $this->currency($companyid[0]);
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $member_id],
            ['company_id', $companyid[0]]
        ])->count();

        try{
            if($userExists > 0){
                DB::beginTransaction();

                // Old balance
                $oldbalance =  \App\Accounts::where([
                    ['user_id', '=', $member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $account =  \App\Accounts::where([
                    ['user_id', '=', $member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->decrement('point', $points);

                $balance =  \App\Accounts::where([
                    ['user_id', '=', $member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['debit'];
                $transaction->type_name = array_keys($this->transtype)[0];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $points;
                $transaction->point_virtual = $points * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->balance_raw = $balance->point;
                $transaction->balance_virtual = $balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $companyid[0];
                $transaction->member_id = $member_id;
                $transaction->reason = $reason;
                $transaction->trans_by = json_encode($transby);
                $transaction->save();
                 //dd($transaction);

                //Check if a total record for the member exists
                $totalExists =  \App\MemberTotal::where(
                    [
                        ['member_id', $member_id],
                        ['company_id', $companyid[0]]
                    ]
                )->count();

                //Check if total for the member exists before updating the user's total credit
                if ($totalExists > 0){
                    //Update totals if a record exists
                    \App\MemberTotal::where(
                        [
                            ['member_id', $member_id],
                            ['company_id', $companyid[0]]
                        ]
                    )->increment('credit', $points);
                }else{
                    //Create a new record if it doesnt exist
                    \App\MemberTotal::firstOrCreate(
                        [
                            'member_id' => $member_id,
                            'company_id' => $companyid[0],
                            'credit' => $points
                        ]
                    );
                }

                DB::commit();

                if($account){
                    $customersAccount = \App\User::where('users.id', "=", $member_id)
                        ->select(
                            'users.firstname',
                            'users.lastname',
                            'users.email',
                            'users.company_id',
                            'users.id as member_id',
                            'users.member_id as member_number'
                        )
                        ->first();

                    $response = [
                        'firstname' => $customersAccount->firstname,
                        'lastname' => $customersAccount->lastname,
                        'email' => $customersAccount->email,
                        'company_id' => $customersAccount->company_id,
                        'member_id' => $customersAccount->member_id,
                        'debit_raw' => $points,
                        'debit_virtual' => $points * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'member_number' => $customersAccount->member_id,
                        'balance_raw' => $transaction->balance_raw,
                        'balance_virtual' => $transaction->balance_virtual,
                        'balance_raw' => $transaction->balance_raw,
                        'old_balance_raw' => $oldbalance->point,
                        'balance_virtual' => $transaction->balance_virtual,
                        'old_balance_virtual' => $oldbalance->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'type_name' => $transaction->type_name,
                    ];
                    return response()->json(['data' => $response, 'status' => '200'], 200);
                }else{
                    return response()->json(['data' => 'Failed to update Account', 'status' => '200'], 200);
                }

            }else{
                return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);
            }

        }catch (\Exception $e){
            DB::rollBack();
            return response()->json(['data' => "Failed to Debit Member",'reason' => $e->getMessage(), 'status' => '400'], 400);
        }
    }
    
    public function MemberChangePassword($member_id)
    {

        $password = str_random(8);
        
        try {

            $change_password = DB::table('users')->where('id', $member_id)
                ->update(
                    [
                        'password' => Hash::make($password)

                    ]
                );

            if($change_password){

                return $password;
            } else {
                return ['data' => ' Error changing password', 'status' => 0];
            }
        }catch (\Exception $e) {


            return ['data' => "An Error Occurred. Could not update account", 'reason' => $e->getMessage(), 'status' => '0'];
        }

    }
}