<?PHP
namespace App\Http\Controllers;
use App\Order;
use App\Repository\OrderRepository;
use Illuminate\Http\Request;
use App\Cart;
use Validator;
use App\Company;
use App\User;
use DB;
use App\Traits\SettingTrait;
use App\Traits\Transaction;
use App\Traits\Email;

class OrderController extends Controller
{
    use SettingTrait, Transaction, Email;

    private $orderstatus = [
        'cancelled' => 0,
        'shipped' => 2,
        'delivered' => 3,
        'Processing' => 4,
        'expired' => 5,
        'pickedup' => 8
    ];


    public $orderRepository;

    public function __construct(OrderRepository $OrderRepository)
    {
        $this->orderRepository = $OrderRepository;
    }


    public function ordersReportByDate(Request $request){

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(
                [
                    'data' => $validator->messages(),
                    'message' => 'Fields missing',
                    'status' => '400',
                ], 400);
        }

        $company = Company::where('client_number', $request->client_number);

        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->select('id')->first();

        $orders = [];
        if ($request->has('member_ids')){
            if(!is_array($request->input('member_ids'))){
                return response()->json(
                    [
                        'data' => [],
                        'message' => 'member_ids has to be an array',
                        'status' => '400',
                    ], 400);
            }
            $orders = \App\Order::where([
                ['company_id', '=', $company->id]
            ])
                ->whereIn('user_id', $request['member_ids']);
        }
        elseif ($request->has('member_id')) {
            $orders = \App\Order::where([
                ['company_id', '=', $company->id]
            ])
                ->where('user_id', $request['member_id']);
        }else{
            $orders = \App\Order::where([
                ['company_id', '=', $company->id]
            ]);
            $data = $this->orderRepository->ordersReportNoDate( $company->id, $orders);
            return response()->json(['data' => $data, 'message' => 'Data loaded', 'status' => '200'], 200);
        }

        if ($request->has('from_date') and $request->has('to_date')) {

            $data = $this->orderRepository->ordersReportByDate($request, $company->id, $orders);

        }else{
            $data = $this->orderRepository->ordersReportNoDate( $company->id, $orders);
        }

        if(!$data){
            return response()->json(
                [
                    'data' => [],
                    'message' => 'No data available',
                    'status' => '400'
                ], 400);
        }

        return response()->json(['data' => $data, 'message' => 'Data loaded', 'status' => '200'], 200);
    }

    public function getCustomerOrder(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();


        if($userExists > 0) {
            $transactions = \App\Order::where(
                [
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0]]
                ]
            )->get();
            return response()->json(['data' => $transactions, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);

        }

    }

    public function getOrderItems(Request $request)
    {
        //Validate request
        $validator = Validator::make($request->all(), [
            'order_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages()], 400);
        }
        //End validate request

        $data = [];

        $orderExists = \App\Order::where('id', '=', $request->order_id)
            ->count();
        if($orderExists < 1){
            return response()->json(['data' => 'Order ID not found', 'status' => '400'], 400);
        }

        $accounts = \App\Order::where('id', '=', $request->order_id)
            ->first();
        
        $currency = $this->currency($accounts->company_id);

        //Shipping address
        $address = null;
        $member = null;
        
        if(isset($accounts->pluck('is_shipping')[0])) {
            if ($accounts->is_shipping === 1) {
                $address = \App\ShippingAddress::where('id', head($accounts->pluck('address_id'))[0])->first();
            }
            $member = User::select('member_id','firstname','lastname')->where('id',$accounts->user_id)->get()->toArray();
        }

        if($accounts->success){
            foreach ($accounts->success as $item){
                $cartitem = \App\Cart::
                where('cart.id',$item['ref_no'])
//                    ->leftJoin('shipping_address', 'cart.delivery_location', '=', 'shipping_address.id')
                    ->first();
                    
                array_push($data,$cartitem);
            }
        }


        if(count($accounts->fail)){
            if(isset($accounts->fail['reference'])) {
                foreach ($accounts->fail['reference'] as $item){
                    $cartitem = \App\Cart::
                    where('cart.id',$item)
//                        ->leftJoin('shipping_address', 'cart.delivery_location', '=', 'shipping_address.id')
                        ->get();
                    array_push($data,(object)head($cartitem->toArray()));
                }
            }
            
        }

        $response = [];
         
        foreach ($data as $item){
           
            $response[] =
            [
            'order_id' => $item->id,
            'user_id' => $item->user_id,
            'company_id' => $item->company_id,
            'name' => $item->name,
            'delivery_type' => $item->delivery_type,
            'delivery_type_name' => $item->delivery_type == 2 ? "Delivery" : $item->delivery_type == 1 ? "Pick up" : "E-Channels",
            'pickup_location_name' => $item->pickup_location_name,
            'qty' => $item->qty,
            'price' => $item->price,
            'virtual_price' => $item->price * (!$currency->is_currency_fixed ? $currency->rate : 1),
            'voucher' => $item->voucher,
            'signature' => $item->signature,
            'image' => $item->image,
            'status' => $item->status,
            'status_text' => array_flip($this->cartstatus)[$item->status],
            'created_at' => $item->created_at,
            'updated_at' => $item->updated_at
            ];
            
            
        }
        

        return response()->json([
            'data' => $response,
//            'address' => $address,
                'member' => $member[0],
            'status' => '200'
        ], 200);
    }

    public function getCompanyOrder(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $company = Company::where('client_number', $request->client_number)->select('id');

        if($company->exists()){
            return response()->json(['data' => 'company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();
        $data = [];
        $data['company'] = collect($company)->toJson();

        try {

//            if($request->has('from_date') && $request->has('to_date') ) {
//                $transactions = $this->orderRepository->ordersReportByDate($request, $company->id);
//            }else{
                $transactions = \App\Order::where('orders.company_id', '=', $company->id)
                ->join('shipping_address', 'orders.address_id', '=', 'shipping_address.id')
                ->join('company', 'orders.company_id', '=', 'company.id')
                ->select(
                    'orders.id',
                    'orders.sub_total_cost',
                    'orders.is_shipping',
                    'orders.shipping_cost',
                    'orders.success',
                    'orders.fail',
                    'orders.created_at as date'
                )
                ->get();
            $data['details'] =  $transactions;
            return response()->json(['data' => $data, 'status' => '200'], 200);
        } catch (\Exception $e) {
            return response()->json(['data' => 'An Error Occurred. Try again.' . $e->getMessage(), 'status' => '400'], 400);
        }
    }

public function get_shipping_address($order_no){

        $address = \App\Order::select('address','state_name','city_name')
        ->where('order_no', '=', $order_no)
            ->leftjoin('shipping_address', 'orders.address_id', '=', 'shipping_address.id')
            ->leftjoin('states', 'shipping_address.state_id', '=', 'states.state_id')
            ->leftjoin('cities', 'shipping_address.city_id', '=', 'cities.city_id')
            ->get()->first();

        if($address!=null){
            //dd($address);
            $address_array = $address->toArray();
            $address_string = $address_array['address'].", ".ucwords(strtolower($address_array['city_name'])).", ".ucwords(strtolower($address_array['state_name'])).",  Nigeria";
               return $address_string;
        } else {

            return null;
        }

    }
    
    public function changeStatus(Request $request){
//        $email_payload = [
//            'first_name' => "Victor",
//            'email' => "victor@loyaltysolutionsnigeria.com",
//            'details' => "<h1>Hello test email after Partner Portal Processing</h1>",
//            'member_no' => 1111
//        ];
//
//
//        return $this->sendStatusChangeEmail(3, "4904GEGFF6IFGE", $email_payload);



        $report = [];
        $check_status = true;

        try {

            $validator = Validator::make($request->all(), [
                'orders' => 'required',
                'status' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }
            }
            //Validate status
            if($request->status == 5 || $request->status == 0){
                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
            }

            $statusstring = array_flip($this->orderstatus)[$request->status]; //Get status string value

            //Start orders loop
            foreach($request->orders as $order) {
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];

                foreach ($report as $key => $item){
                    if($item['order_no']== $order['order_no']){

                        if($request->status == 2 || $request->status == 3){

                            $order['address'] = $this->get_shipping_address($item['order_no']);
                            
                        } else {

                            $order['address'] = null;
                        }
                        //Start vouchers loop
                foreach($order['vouchers'] as $voucher) {
                    if(!$this->itemExist($voucher)){
                        continue;
                    }
                    //Rules
                    //1. No further action can be carried out if Item has already been cancelled or expired
                    //2. No further action if item does not exist
                    //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                    //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                    if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['action']
                        && $this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']
                        && !$this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {

                        $statusvalue = $this->orderstatus[$statusstring]; //Get status number value

//                        //Change item status
                        DB::table('cart')->where('voucher', $voucher)
                            ->update(['status' => $statusvalue]);
                        $check_status = false;
                            $report[$key]['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Item status has been changed to ' . ucfirst($statusstring)
                                ];
                    }
                    //Failed to proceed. Add to Errors array
                    if($check_status) {
                        $message = '';
                        if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['action']) {
                            $message = 'Item status is already set to ' . ucfirst($statusstring);
                        } else if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']) {
                            $message = 'Item does not exist in the records';
                        } else if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {
                            $message = $this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['message'];
                        }
                        $report[$key]['errors'][] = (object)[
                            'vouchers' => [$voucher],
                            'status' => '400',
                            'message' => $message
                        ];
                    }
                    $check_status = true;
                }
                //End vouchers loop
                    }
                }
                if(count($report[$key]['success']) > 0){
                    $receipt = $this->buildSuccessDetails($report[$key]['success']);
                    $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                    $details = $this->buildTable($receipt,$order['address'], $order['order_no']);
                        $email_payload = [
                            'first_name' => $user_data->firstname,
                            'email' => $user_data->email,
                            'details' => $details,
                            'order_no' => $order['order_no'],
                            'member_no' => $user_data->member_id,
                            'branch_name' => $receipt[0]['pickup_location_name']
                        ];
                        if($request->status !=2){
                        $this->sendStatusChangeEmail($request->status, $token->token, $email_payload);
                        }
                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
            }

    }

    public function expireOrder(Request $request){
        $report = [];
        $check_status = true;

        try {

            $validator = Validator::make($request->all(), [
                'orders' => 'required',
                'status' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }
            }
            //Validate status
            if($request->status != 5){
                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
            }

            $statusstring = array_flip($this->orderstatus)[$request->status]; //Get status string value

            //Start orders loop
            foreach($request->orders as $order) {
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];

                foreach ($report as $key => $item){
                    if($item['order_no']== $order['order_no']){
                        //Start vouchers loop
                        foreach($order['vouchers'] as $voucher) {
                            if(!$this->itemExist($voucher)){
                                $report['order_errors'][] = (object)[
                                    'order_no' => $order['order_no'],
                                    'vouchers' => $order['vouchers'],
                                    'status' => '400',
                                    'message' => 'Items do not belong to the same Order '.$order['order_no']
                                ];
                                continue;
                            }
                            //Rules
                            //1. No further action can be carried out if Item has already been cancelled or expired
                            //2. No further action if item does not exist
                            //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                            //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                            if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['action']
                                && $this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']
                                && !$this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {

                                $statusvalue = $this->orderstatus[$statusstring]; //Get status number value

//                        //Change item status
                                DB::table('cart')->where('voucher', $voucher)
                                    ->update(['status' => $statusvalue]);
                                $check_status = false;
                                $report[$key]['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Item status has been changed to ' . ucfirst($statusstring)
                                ];
                            }
                            //Failed to proceed. Add to Errors array
                            if($check_status) {
                                $message = '';
                                if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['action']) {
                                    $message = 'Item status is already set to ' . ucfirst($statusstring);
                                } else if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']) {
                                    $message = 'Item does not exist in the records';
                                } else if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {
                                    $message = $this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['message'];
                                }
                                $report[$key]['errors'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '400',
                                    'message' => $message
                                ];
                            }
                            $check_status = true;
                        }

                        if(count($report[$key]['success']) > 0){
                            $receipt = $this->buildSuccessDetails($report[$key]['success']);
                            $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id')->first();
                            $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id','member_id')->first();
                            $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                            $details = $this->buildTable($receipt,null, $order['order_no']);

                            //Refund
                            try {
                                $refund = $this->refundToAccountWithoutDelivery($order['order_no'], collect($report[$key]['success'])->pluck('vouchers')->flatten()->toArray());
                                if($refund['status']){
                                    $email_payload = [
                                        'first_name' => $user_data->firstname,
                                        'email' => $user_data->email,
                                        'details' => $details,
                                        'current_balance' => $refund['balance'],
                                        'refund_amount' => $refund['refund_amount'],
                                        'member_no' => $user_data->member_id,
                                        'order_no' => $order['order_no']
                                    ];
                                    //Send Email
                                    $email = $this->sendStatusChangeEmail($request->status, $token->token, $email_payload);

                                    $report['email_log'][] = (object)[
                                        'message' => $email,
                                        'order_no' => $order['order_no'],
                                        'status' => '400'
                                    ];
                                }else{
                                    //Revert status
                                    foreach (collect($report[$key]['success'])->pluck('vouchers')->flatten()->toArray() as $voucher) {
                                        DB::table('cart')->where('voucher', $voucher)
                                            ->update(['status' => 1]);
                                    }
                                    $report['refund_fail_errors'][] = (object)[
                                        'message' => $refund['message'],
                                        'order_no' => $order['order_no'],
                                        'status' => '400'
                                    ];
                                }
                            }
                            catch (\Exception $e) {
                                $report['refund_exception_errors'][] = (object)[
                                    'message' => $e->getMessage(),
                                    'error_trace' => $e->getTraceAsString(),
                                    'order_no' => $order['order_no'],
                                    'status' => '400'
                                ];
                            }

                        }
                        //End vouchers loop
                    }
                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function cancelOrder(Request $request){
//        $cart = \App\Cart::
//        where('user_id', '=', 1)
//            ->where('company_id', '=', 2)
//            ->whereIn('voucher', ['SP212791H11J7D1'])
//            ->select(DB::raw('sum(price*qty) as totalprice'))
//            ->first();
//        dd($cart->totalprice);

        $report = [];
        $check_status = true;

        try {
            $validator = Validator::make($request->all(), [
                'orders' => 'required',
                'status' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }

            //validate orders array
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }if (!isset($order['reason'])) {
                    return response()->json(['data' => 'Missing field - orders->reason', 'status' => '400'], 400);
                }
            }
            //Validate status
            if($request->status == 2 || $request->status == 3 || $request->status == 8|| $request->status == 5){
                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
            }

            $statusstring = array_flip($this->orderstatus)[$request->status]; //Get status string value

            //Start orders loop
            foreach($request->orders as $order) {
                //Define base values for report array
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];
                //Check if all vouchers belong to the same order
                $itemsBelongToSameOrder = $this->itemsBelongToSameOrder($order['order_no'], $order['vouchers'])['status'];

                if(!$itemsBelongToSameOrder){
                    $report['order_errors'][] = (object)[
                        'order_no' => $order['order_no'],
                        'vouchers' => $order['vouchers'],
                        'status' => '400',
                        'message' => 'Items do not belong to the same Order '.$order['order_no']
                    ];
                    continue;
                };

                foreach ($report as $key => $item){
                    if($item['order_no']== $order['order_no']){

                        //Start vouchers loop
                        foreach($order['vouchers'] as $voucher) {
                            if(!$this->itemExist($voucher)){
                                continue;
                            }
                            //Rules
                            //1. No further action can be carried out if Item has already been cancelled or expired
                            //2. No further action if item does not exist
                            //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                            //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                            if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['action']
                                && $this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']
                                && !$this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {

                                $statusvalue = $this->orderstatus[$statusstring]; //Get status number value

                                  //Change item status
                                DB::table('cart')->where('voucher', $voucher)
                                    ->update(['status' => $statusvalue]);
                                $check_status = false;
                                $report[$key]['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Item status has been changed to ' . ucfirst($statusstring)
                                ];
                            }
                            //Failed to proceed. Add to Errors array
                            if($check_status) {
                                $message = '';
                                if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['action']) {
                                    $message = 'Item status is already set to ' . ucfirst($statusstring);
                                } else if (!$this->proceedOrderAction($voucher, $request->status, $statusstring)['exist']) {
                                    $message = 'Item does not exist in the records';
                                } else if ($this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['status']) {
                                    $message = $this->proceedOrderAction($voucher, $request->status, $statusstring)['wrongstatus']['message'];
                                }
                                $report[$key]['errors'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '400',
                                    'message' => $message
                                ];
                            }
                            $check_status = true;
                        }

                        //End vouchers loop
                    }
                }
                if(count($report[$key]['success']) > 0){
                    $receipt = $this->buildSuccessDetails($report[$key]['success']);
                    $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id', 'shipping_cost')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                    $details = $this->buildTable($receipt,null, $order['order_no']);

                    $filterd_vouchers = [];
                    foreach ($report[$key]['success'] as $voucher) {
                        foreach ($voucher->vouchers as $voucher_2) {
                            array_push($filterd_vouchers, $voucher_2);
                        }
                    }

                    //Refund
                    try {
                        $new_delivery_cost =  $order['delivery_cost'];
                        //$new_delivery_cost = $order_data->shipping_cost - $order['delivery_cost'];


                        $refund = $this->refundToAccountPlusDelivery($order['order_no'], $filterd_vouchers, $new_delivery_cost, $order['reason']);
                        if($refund['status']){
                            $email_payload = [
                                'first_name' => $user_data->firstname,
                                'email' => $user_data->email,
                                'details' => $details,
                                'current_balance' => $refund['balance'],
                                'refund_amount' => $refund['refund_amount'],
                                'member_no' => $user_data->member_id,
                                'order_no' => $order['order_no'],
                                'item_name' => $receipt[0]['name']
                            ];

                            //Send Email
                            $email = $this->sendStatusChangeEmail($request->status, $token->token, $email_payload);
                            $report['email_log'][] = (object)[
                                'message' => $email,
                                'order_no' => $order['order_no'],
                                'status' => '200'
                            ];
                        }else{
                            //Revert status
                            foreach ($filterd_vouchers as $voucher) {
                                DB::table('cart')->where('voucher', $voucher)
                                    ->update(['status' => 1]);
                            }
                            $report['refund_fail_errors'][] = (object)[
                                'message' => $refund['message'],
                                'order_no' => $order['order_no'],
                                'status' => '400'
                            ];
                        }
                    }catch (\Exception $e) {
                        $report['refund_exception_errors'][] = (object)[
                            'message' => $e->getMessage(),
                            'error_trace' => $e->getTraceAsString(),
                            'order_no' => $order['order_no'],
                            'status' => '400'
                        ];
                    }

                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }

    }

    public function expireSevenDays(Request $request){
        $report = [];

        try {

            $validator = Validator::make($request->all(), [
                'orders' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }
                if (!isset($order['expiry_date'])) {
                    return response()->json(['data' => 'Missing field - orders->expiry_date', 'status' => '400'], 400);
                }
            }


            //Start orders loop
            foreach($request->orders as $order) {
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];
                if(!$this->orderNoExist($order['order_no']) ){
                    $report['order_errors'][] = (object)[
                        'order_no' => $order['order_no'],
                        'status' => '400',
                        'message' => "Order ID does not exist"
                    ];
                    continue;
                }


                foreach ($report as $key => $item){
                    if($item['order_no']== $order['order_no']){
                        //Start vouchers loop
                        foreach($order['vouchers'] as $voucher) {
                            if(!$this->itemExist($voucher)){
                                $report[$key]['errors'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '400',
                                    'message' => "Voucher does not exist"
                                ];
                                continue;
                            }

                                $report[$key]['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Voucher processed'
                                ];

                            }
                        //End vouchers loop
                    }
                }

                if(count($report[$key]['success']) > 0){

                    $receipt = $this->buildSuccessDetails($report[$key]['success']);
                    $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id','member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                    $details = $this->buildExpireTable($receipt,null, $order['order_no']);

                    $email_payload = [
                        'first_name' => $user_data->firstname,
                        'email' => $user_data->email,
                        'details' => $details,
                        'expiry_date' => $order['expiry_date'],
                        'member_no' => $user_data->member_id
                    ];

                    $email = $this->sendStatusExpireEmail_7($token->token, $email_payload);

                    $report['email_log'][] = (object)[
                        'message' => $email,
                        'order_no' => $order['order_no'],
                        'status' => '400'
                    ];

                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(). $e->getLine(), 'status' => '400'], 400);
        }
    }

    public function expireTwoDays(Request $request){
        $report = [];

        try {

            $validator = Validator::make($request->all(), [
                'orders' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }
                if (!isset($order['expiry_date'])) {
                    return response()->json(['data' => 'Missing field - orders->expiry_date', 'status' => '400'], 400);
                }
            }


            //Start orders loop
            foreach($request->orders as $order) {
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];

                if(!$this->orderNoExist($order['order_no']) ){
                    $report['order_errors'][] = (object)[
                        'order_no' => $order['order_no'],
                        'status' => '400',
                        'message' => "Voucher does not exist"
                    ];
                    continue;
                }

                foreach ($report as $key => $item){
                    if($item['order_no']== $order['order_no']){
                        //Start vouchers loop
                        foreach($order['vouchers'] as $voucher) {
                            if(!$this->itemExist($voucher)){
                                $report[$key]['errors'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '400',
                                    'message' => "Voucher does not exist"
                                ];
                                continue;
                            }

                                $report[$key]['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Voucher processed'
                                ];

                            }
                        //End vouchers loop
                    }
                }

                if(count($report[$key]['success']) > 0){
                    $receipt = $this->buildSuccessDetails($report[$key]['success']);
                    $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                    $details = $this->buildExpireTable($receipt,null, $order['order_no']);


                    $email_payload = [
                        'first_name' => $user_data->firstname,
                        'email' => $user_data->email,
                        'details' => $details,
                        'expiry_date' => $order['expiry_date'],
                        'member_no' => $user_data->member_id
                    ];

                    $email = $this->sendStatusExpireEmail_2($token->token, $email_payload);

                    $report['email_log'][] = (object)[
                        'message' => $email,
                        'order_no' => $order['order_no'],
                        'status' => '400'
                    ];

                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }
    }

    private function itemExist($voucher)
    {
        $itemExists = Cart::
        where('voucher', $voucher)->count();
        if ($itemExists < 1) {
            return false;
        } else {
            return true;
        }
    }

    private function orderNoExist($order_no){
        $orderNoExists = Order::
    where('order_no', $order_no)->count();
    if($orderNoExists < 1){
        return false;
    }else{
        return true;
    }
    }

    private function proceedOrderAction($voucher, $status, $statusstring){
        $result = [];

        $item = DB::table('cart')->where('voucher', $voucher)
            ->select('status', 'delivery_type')
            ->first();

        if($item->status == $status){
            $result['action'] = false;
            $result['status'] = $item->status;
        }
        else{
            $result['action'] = true;
            $result['status'] = $item->status;
        }

        $itemExists = Cart::
            where('voucher', $voucher)->count();
            if($itemExists < 1){
                $result['exist'] = false;
            }else{
                $result['exist'] = true;
            }

            if($item->delivery_type == 1){
                if($status == 3 || $status == 2){
                    $result['wrongstatus'] = ['status' => true, 'message' => 'Wrong status. Item status cannot be changed to ' . ucfirst($statusstring) . ' because it is a Pickup item'];
                }else{
                    $result['wrongstatus'] = ['status' => false, 'message' => ''];
                }
            }

            if($item->delivery_type == 2){
                if($status == 5 || $status == 8){
                    $result['wrongstatus'] = ['status' => true, 'message' => 'Wrong status. Item status cannot be changed to ' . ucfirst($statusstring) . ' because it is a Delivery item'];
                }else{
                    $result['wrongstatus'] = ['status' => false, 'message' => ''];
                }
            }

            if($item->delivery_type != 2 && $item->delivery_type != 1){
                $result['wrongstatus'] = ['status' => false, 'message' => ''];
            }
            return $result;
    }

    private function itemsBelongToSameOrder($orderid, $vouchers){
        $response = [];
        $data = [];
        $firstFilter = \App\Order::where('order_no', $orderid)->select('success')->pluck('success');

        $secondFilter = collect($firstFilter)->reject(function($item){
            return collect($item)->flatten()->isEmpty();
        });

        if(count($secondFilter) < 1){
            $response['status'] = false;
            return $response;
        }

        $orderVouchersCount = collect(json_decode(head($secondFilter)[0]))->pluck('voucher_code')->count();

        foreach (json_decode(head($secondFilter)[0]) as $order) {
                foreach ($vouchers as $voucher) {
                    if ($order->voucher_code == $voucher) {
                        array_push($data, $order);
                    }
                }
        };
        $newvouchers_data = collect($data)->pluck('voucher_code');

        if($newvouchers_data->count() === count($vouchers)){
            $response['status'] = true;
        }else{
            $response['status'] = false;
        }

        return $response;
    }

    private function getDeliveryItems($vouchers){
        $response = ['status' => null, 'data' => []];

        $delivery_items = [];

            foreach ($vouchers as $voucher) {
                $result = DB::table('cart')
                    ->where('voucher', $voucher)
                    ->where('delivery_type', 2)
                    ->whereNotIn('status', [2, 3, 0])
                    ->select('delivery_location', 'voucher')
                    ->first();
                    array_push($delivery_items, $result);
            }

        $delivery_location = collect($delivery_items)->filter()->flatten()->pluck('delivery_location');
        $voucher = collect($delivery_items)->filter()->flatten()->pluck('voucher');
            if(count($delivery_location)){
                $response['status'] = true;
                $response['delivery_location'] = $delivery_location;
                $response['voucher'] = $voucher;
            }else{
                $response['status'] = false;
            }
            return $response;
    }

    private function getdeliveryprice($vouchers, $address_id)
    {
        $curlService = new \Ixudra\Curl\CurlService();
        $product_details = [];
        $cart = [];

            $address = \App\ShippingAddress::findOrFail($address_id);
            if ($address) {
                $cart['state_id'] = $address->state_id;
                $cart['city_id'] = $address->city_id;
            }

            $pendingcart = DB::table('cart')
                ->whereIn('voucher', $vouchers)
                ->get();

            if($pendingcart){
                foreach ($pendingcart as $item){
                    $product_details[] = [
                        'signature' => $item->signature,
                        'quantity' => $item->qty
                    ];
                }
                $cart['product_details'] = json_encode($product_details);
            }

        try{
            $response = $curlService->to($this->baseUrl.'delivery_price')
                ->withHeader('token: 1200')
                ->withData($cart)
                ->asJsonResponse()
                ->post();
            if($response) {
                if($response->status == 0){
                    return response()->json(['data' => $response->message, 'status' => '400'], 400);
                }
                return response()->json(['data' => $response, 'status' => '200'], 200);
            }

        }catch (\Exception $e){
            return response()->json(['data' => 'An error occurred' .$e->getMessage(), 'status' => '400'], 400);
        }
    }

    private function sortItemsByOrder($vouchers){
        $response = [];
        $data = [];
        $firstFilter = [];
        foreach (\App\Order::select('success', 'user_id')->cursor() as $order) {
            array_push($firstFilter, $order);
        }

        $secondFilter = collect($firstFilter)->reject(function($item){
            return collect($item)->flatten()->isEmpty();
        });
        $ordersArray = collect($secondFilter)->map(function($item){
            return (object)collect($item)->flatten();
        });

        foreach ($ordersArray->toArray() as $order) {
            if (count($order) > 1) {
                foreach ($vouchers as $voucher) {
                    if ($order[1] == $voucher) {
                        array_push($data, $order);
                    }
                }
            }
        };

        if(count($data) < 1){
            $response['status'] = false;
        }else if(count($data) == 1){
            $response['status'] = true;
            $response['data'] = $data;
            $response['multiple'] = false;
        }else if(count($data) > 1){
            $response['status'] = true;
            $response['data'] = $data;
            $response['multiple'] = true;
        }

        return $response;
        }

public function voucherStatus($voucher_no){

    $voucherStatus = Cart::select('name', 'status')
        ->where('voucher', $voucher_no)
        ->first()->toArray();

    if($voucherStatus){

        return $voucherStatus;

    } else {

        return "Voucher Not found";
    }


}

public function OrderVoucherExists($order_no, $voucher)
{

    $voucherExists = Order::select('order_no', 'success')
        ->where('order_no', $order_no)
        ->first();



    if ($voucherExists) {
            $ordercolumn = $voucherExists->success;

            if(is_array($ordercolumn)){

                foreach ($ordercolumn as $c_data){
                    
                    if($c_data['voucher_code'] == $voucher){

                        $check_status = $this->voucherStatus($voucher);
                        
                        if(is_array($check_status)){
                            if($check_status['status'] == 2){

                                return 1;
                            } else {

                                return "Item not shipped";
                            }

                        } else {

                            return $check_status;
                        }
                        
                    }

                }
                return "Invalid Voucher";
            } else {

                return "Invalid Format";
            }
    } else {
        return "Invalid Order";
    }
}




public function confirmShipped(Request $request){


        $report = [];


        try {

            $validator = Validator::make($request->all(), [
                'orders' => 'required',
                'status' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }
            foreach($request->orders as $order) {
                if (!isset($order['order_no'])) {
                    return response()->json(['data' => 'Missing field - orders->order_no', 'status' => '400'], 400);
                }
                if (!isset($order['vouchers'])) {
                    return response()->json(['data' => 'Missing field - orders->vouchers', 'status' => '400'], 400);
                }
            }
            //Validate status
            if($request->status != 2){
                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
            }

            $statusstring = array_flip($this->orderstatus)[$request->status]; //Get status string value

            //Start orders loop
            foreach($request->orders as $order) {
                $report[] = [ 'order_no' => $order['order_no'], 'errors' => [], 'success' => [] ];

                foreach ($report as $key => $item){
                    //dd($order['order_no']);
                    if($item['order_no']== $order['order_no']){

                            $order['address'] = $this->get_shipping_address($item['order_no']);

                        foreach($order['vouchers'] as $voucher) {
                            $check_voucher =$this->OrderVoucherExists($item['order_no'], $voucher);
//                            if(!$this->itemExist($voucher)){
//                                continue;
//                            }
                            //Rules
                            if($check_voucher == 1) {
                                //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                                //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error

                                    $report[$key]['success'][] = (object)[
                                        'vouchers' => [$voucher],
                                        'status' => '200',
                                        'message' => 'Item status has been changed to ' . ucfirst($statusstring)
                                    ];



                            } else {

                                $report[$key]['errors'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '400',
                                    'message' => $check_voucher
                                ];
                            }
                        }
                        //End vouchers loop
                    }
                }
                if(count($report[$key]['success']) > 0){
                    $receipt = $this->buildSuccessDetails($report[$key]['success']);
                    $order_data =  \App\Order::where('order_no', $order['order_no'])->select('user_id')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                    $details = $this->buildTable($receipt,$order['address'], $order['order_no']);
                    $email_payload = [
                        'first_name' => $user_data->firstname,
                        'email' => $user_data->email,
                        'details' => $details,
                        'order_no' => $order['order_no'],
                        'member_no' => $user_data->member_id,
                        'branch_name' => $receipt[0]['pickup_location_name']
                    ];

                 

                    if($request->status == 2) {
                        $this->sendStatusChangeEmail($request->status, $token->token, $email_payload);
                    }
                }
            }
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }

    }
    
    
    private function pendingItem($ref_no, $company_id){

        $refNoPending = Cart::
        where('id', $ref_no)
        ->where ('status', 6)
        ->where ('delivery_type', 5)
        ->where ('company_id', $company_id);
            //->whereIn('delivery_type', [3,4,5,6])
            //->count();
        //dd($company_id);
        if(!$refNoPending-> exists()){
            //dd($ref_no);
            return ['data' => 'Pending item not found', 'status' => '400'];
        }else{
    
            //dd("No");
            $refNoPending = $refNoPending->select('id','user_id','name','qty','price','company_id')->first();
            $verify = $refNoPending;
            $verify['status']=200;
            return $verify;
    
        }
    }
    
    private function verifyClientNo($client_number){
        $company = Company::where('client_number', $client_number);
    
        if(!$company->exists()){
            return ['data' => 'Company does not exist', 'status' => '400'];
        } else {
    
            $company = $company->select('id')->first();
            $verify = $company->toArray();
            $verify['status']=200;
            return $verify;
        }
    }
    
    private function updateCartStatus ($ref_no, $voucher_code, $status){
    
        $cart = Cart::where ('id', $ref_no)
            ->update(['status' => $status],['voucher' => $voucher_code]);
    
        if($cart){
    
            return true;
        } else {
    
            return false;
        }
    }
    public function resolvePending(Request $request){
    
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'ref_no' => 'required',
            'status' => 'required',
            'voucher_no'=>'required',
            'order_no'=>'required'
        ]);
        if ($validator->fails()) {
            return response()->json(
                [
                    'data' => $validator->messages(),
                    'message' => 'Parameters missing',
                    'status' => '400',
                ], 400);
        } else {
    
            $verifyClient = $this->verifyClientNo($request->client_number);
            if ($verifyClient['status'] == 400) {
    
            
                return $verifyClient;
            } else{
    
                $company_id =  $verifyClient['id'];
    
                $verifyCart = $this->pendingItem($request->ref_no, $company_id);
                
                
                if($verifyCart['status'] == 200){
    
                    if ($request->status == 0){
    
                        $changeStatus = $this->updateCartStatus($request->ref_no, $request->voucher_no, 0);
                        
                        if ($changeStatus){
                            $refund = $this->refundpendingAccount($verifyCart['user_id'],$verifyCart['name'],$verifyCart['qty'], $verifyCart['price'],$verifyCart['company_id'],$verifyCart['id'],$request->voucher_no, $request->order_no);
                        //dd($refund);
                                if ($refund['status']){
    
                                    return response()->json(['data' => $refund, 'status' => '200'], 200);
                                } else {
    
                                    return response()->json(['data' => $refund['message'], 'status' => '400'], 400);
                                }
    
    
    
    
                        } else {
    
                            return response()->json(
                                [
    
                                    'message' => 'Error updating status',
                                    'status' => '400',
                                ], 400);
                        }
    
                    } elseif ($request->status == 1){
    
                        $changeStatus = $this->updateCartStatus($request->ref_no, $request->voucher_no, 3);
                        if ($changeStatus){
                            
                            $cost = $this->completependingAccount($verifyCart['user_id'],$verifyCart['name'],$verifyCart['qty'], $verifyCart['price'],$verifyCart['company_id'],$verifyCart['id'],$request->voucher_no, $request->order_no);
                            //dd($cost);
                            
                            if ($cost['status']){
                                 return response()->json(['data' => $cost, 'status' => '200'], 200);
                              
    
                            } else {
    
                                return response()->json(['data' => $cost['message'], 'status' => '400'], 400);
                            }
    
    
    
    
                        } else {
    
                            return response()->json(
                                [
    
                                    'message' => 'Error updating status',
                                    'status' => '400',
                                ], 400);
                        }
                    } else {
    
    
            return response()->json(
                [
                    'data' => $request->status,
                    'message' => 'Invalid Status sent',
                    'status' => '400',
                ], 400);
    
                    }
                }else{
    
                    return $verifyCart;
                }
    
            }
        }
    
    
    
    }
    
    
    public function get_pending_cart(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(
                [
                    'data' => $validator->messages(),
                    'message' => 'Fields missing',
                    'status' => '400',
                ], 400);
        }

        $company = Company::where('client_number', $request->client_number);

        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->select('id')->first();


        $cartitems = \App\Cart::
        where('company_id',$company->id)
            ->where('delivery_type', 5)
            ->where('status', 6)
            ->get();

        return response()->json(['data' => $cartitems, 'status' => '200'], 200);
    }
    
    
        public function bulk_cancel_order (Request $request){
        $statusstring = "cancelled";
        $check_status = true;
        try {
            $validator = Validator::make($request->all(), [
                'order_no' => 'required',
                'refund_amount' => 'required|numeric',
                'orders' => 'required|array',
                'orders.*.voucher_no' => 'required',
                'orders.*.reason' => 'required',
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
            if (!is_array($request->orders)){
                return response()->json(['data' => 'Invalid data format. orders is not an array.', 'status' => '400'], 400);
            }

            $vouchers = array_column($request->orders, 'voucher_no');
            //return $vouchers;
            $orders = $request->orders;

            $report= array();
            //Check if all vouchers belong to the same order
            $itemsBelongToSameOrder = $this->itemsBelongToSameOrder($request->order_no, $vouchers)['status'];
            //dd($itemsBelongToSameOrder);
            if(!$itemsBelongToSameOrder){
                $report['order_errors'][] = (object)[
                    'order_no' => $request->order_no,
                    'vouchers' => $vouchers,
                    'status' => '400',
                    'message' => 'Items do not belong to the same Order '.$request->order_no
                ];

            };

            //dd($itemsBelongToSameOrder);
            //Start vouchers loop
            foreach($orders as $order) {
                if(!$this->itemExist($order['voucher_no'])){
                    continue;
                }
                //Rules
                //1. No further action can be carried out if Item has already been cancelled or expired
                //2. No further action if item does not exist
                //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                if ($this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['action']
                    && $this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['exist']
                    && !$this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['wrongstatus']['status']) {


                    $statusvalue = $this->orderstatus[$statusstring]; //Get status number value
//dd($statusvalue);
                    //Change item status
                    DB::table('cart')->where('voucher', $order['voucher_no'])
                        ->update(['status' => $statusvalue, 'remark' => $order['reason']]);
                    $check_status = false;
                    $report['success'][] = (object)[
                        'vouchers' => [$order['voucher_no']],
                        'status' => '200',
                        'message' => 'Item status has been changed to ' . ucfirst($statusstring)
                    ];
                }
                //Failed to proceed. Add to Errors array
                if($check_status) {
                    $message = '';
                    //dd($voucher);
                    if (!$this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['action']) {
                        $message = 'Item status is already set to ' . ucfirst($statusstring);
                    } else if (!$this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['exist']) {
                        $message = 'Item does not exist in the records';
                    } else if ($this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['wrongstatus']['status']) {
                        $message = $this->proceedOrderAction($order['voucher_no'], 0, $statusstring)['wrongstatus']['message'];
                    }
                    //dd($message);
                    $report['errors'][] = (object)[
                        'vouchers' => [$order['voucher_no']],
                        'status' => '400',
                        'message' => $message
                    ];
                }
                $check_status = true;
            }

            //End vouchers loop
            //dd($report);
            if(count($report['success']) > 0){
                $receipt = $this->buildSuccessDetails($report['success']);
                $order_data =  \App\Order::where('order_no', $request->order_no)->select('user_id', 'shipping_cost')->first();
                $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();
                $details = $this->buildCancelTable($receipt,null, $request->order_no);

//                echo $details;
//                dd();
                $filterd_vouchers = [];
                foreach ($report['success'] as $voucher) {
                    foreach ($voucher->vouchers as $voucher_2) {
                        array_push($filterd_vouchers, $voucher_2);
                    }
                }

                //Refund
                try {
                    $new_delivery_cost =  $request->refund_amount;

//dd($new_delivery_cost);
                    $refund = $this->refundToAccountPlusDelivery($request->order_no, $filterd_vouchers, $new_delivery_cost, "cancelled");
//dd($refund);
                    if($refund['status']){
                        $email_payload = [
                            'first_name' => $user_data->firstname,
                            'email' => $user_data->email,
                            'details' => $details,
                            'current_balance' => $refund['balance'],
                            'refund_amount' => $refund['refund_amount'],
                            'member_no' => $user_data->member_id,
                            'order_no' => $request->order_no,
                            'item_name' => $receipt[0]['name']
                        ];

                        //Send Email
                        $email = $this->sendStatusChangeEmail(0, $token->token, $email_payload);
                        $report['email_log'][] = (object)[
                            'message' => $email,
                            'order_no' => $request->order_no,
                            'status' => '200'
                        ];
                    }else{
                        //Revert status
                        foreach ($filterd_vouchers as $voucher) {
                            DB::table('cart')->where('voucher', $voucher)
                                ->update(['status' => 1]);
                        }
                        $report['refund_fail_errors'][] = (object)[
                            'message' => $refund['message'],
                            'order_no' => $request->order_no,
                            'status' => '400'
                        ];
                    }
                }catch (\Exception $e) {
                    $report['refund_exception_errors'][] = (object)[
                        'message' => $e->getMessage(),
                        'error_trace' => $e->getTraceAsString(),
                        'order_no' => $request->order_no,
                        'status' => '400'
                    ];
                }

            }

            return response()->json(['data' => $report, 'status' => '200'], 200);
        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }
    }
    
    
    public function cancelPickOrder(Request $request){


        $report = [];
        $check_status = true;

        try {
            $validator = Validator::make($request->all(), [
                'cancel' => 'present|array',
                'status' => 'required|int',
                'order_no' => 'required|int',
                'pickup' => 'present|array',
            ]);

            if ($validator->fails()) {
                return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
            }
//            $pick_vouchers = json_decode($request->pickup, true);
//            $cancel_vouchers = json_decode($request->cancel, true);
            $pick_vouchers = $request->pickup;
            $cancel_vouchers = $request->cancel;
            $report = array();
            $report['order_no'] = $request->order_no;
            $report['cancel']['success'] = array();
            $report['cancel']['errors'] = array();
            $report['pickup']['success'] = array();
            $report['pickup']['errors'] = array();

            if (!is_array($pick_vouchers)){
                return response()->json(['data' => 'Invalid pickup data format. vouchers is not an array.', 'status' => '400'], 400);
            }

            if (!is_array($cancel_vouchers)){
                return response()->json(['data' => 'Invalid cancel data format. vouchers is not an array.', 'status' => '400'], 400);
            }

            //validate orders array
            if(count($cancel_vouchers) > 0) {
                foreach ($cancel_vouchers as $voucher) {
                    if (!isset($voucher['voucher_no'])) {
                        return response()->json(['data' => 'Missing field - vouchers->voucher_no', 'status' => '400'], 400);
                    }
                    if (!isset($voucher['reason'])) {
                        return response()->json(['data' => 'Missing field - cancel vouchers->reason', 'status' => '400'], 400);
                    }
                }
            }

            if(count($pick_vouchers) > 0) {
                foreach ($pick_vouchers as $voucher) {
                    if (!isset($voucher['voucher_no'])) {
                        return response()->json(['data' => 'Missing field - vouchers->voucher_no', 'status' => '400'], 400);
                    }

                }
            }
            //Validate status
//            if($request->status == 2 || $request->status == 3 || $request->status == 8|| $request->status == 5){
//                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
//            }
            if($request->status == 12){

                $statusstring1 = "cancelled"; //Get status string value
                $statusstring2 = "pickedup";

                // check if vouchers returned belong to the same order // start
            } else {

                return response()->json(['data' => 'Invalid Status', 'status' => '400'], 400);
            }

            //$statusstring = array_flip($this->orderstatus)[$request->status]; //Get status string value

            //Start orders loop
            //foreach($request->orders as $order) {
                //Define base values for report array


                        //Start vouchers loop

                    if(count($cancel_vouchers) > 0) {
                        $cancel_status = 0;
                        foreach ($cancel_vouchers as $voucher) {
                            //dd($voucher);
                            if (!$this->itemExist($voucher['voucher_no'])) {
                                $report['cancel']['errors'][] = (object)[
                                    'vouchers' => [$voucher['voucher_no']],
                                    'status' => '400',
                                    'message' => $voucher['voucher_no'].' does not exist'
                                ];
                                continue;
                            }
                            //Rules
                            //1. No further action can be carried out if Item has already been cancelled or expired
                            //2. No further action if item does not exist
                            //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                            //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                            if ($this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['action']
                                && $this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['exist']
                                && !$this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['wrongstatus']['status']) {

                                $statusvalue = $this->orderstatus[$statusstring1]; //Get status number value

                                //Change item status
                                DB::table('cart')->where('voucher', $voucher['voucher_no'])
                                    ->update(['status' => $statusvalue]);
                                $check_status = false;
                                $report['cancel']['success'][] = (object)[
                                    'vouchers' => [$voucher['voucher_no']],
                                    'status' => '200',
                                    'message' => 'Item status has been changed to ' . ucfirst($statusstring1)
                                ];
                            }
                            //Failed to proceed. Add to Errors array
                            if ($check_status) {
                                $message = '';
                                if (!$this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['action']) {
                                    $message = 'Item status is already set to ' . ucfirst($statusstring1);
                                } else if (!$this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['exist']) {
                                    $message = 'Item does not exist in the records';
                                } else if ($this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['wrongstatus']['status']) {
                                    $message = $this->proceedOrderAction($voucher['voucher_no'], $cancel_status, $statusstring1)['wrongstatus']['message'];
                                }
                                $report['cancel']['errors'][] = (object)[
                                    'vouchers' => [$voucher['voucher_no']],
                                    'status' => '400',
                                    'message' => $message
                                ];
                            }
                            $check_status = true;
                        }

                        //End vouchers loop cancel
                    }

                    if(count($pick_vouchers) > 0) {
                        $pick_status = 8;
                        //Start pickupvouchers loop
                        foreach ($pick_vouchers as $voucher) {
                            if (!$this->itemExist($voucher['voucher_no'])) {
                                $report['pickup']['errors'][] = (object)[
                                    'vouchers' => [$voucher['voucher_no']],
                                    'status' => '400',
                                    'message' => $voucher['voucher_no'].' does not exist'
                                ];
                                continue;
                            }
                            //Rules
                            //1. No further action can be carried out if Item has already been cancelled or expired
                            //2. No further action if item does not exist
                            //3. Pickup(1) items status cannot be changed to shipped (2) or delivered (3) - Wrong status error
                            //4. Delivery(2) items status cannot be changed to expired (5) or pickedup (8) - Wrong status error
                            if ($this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['action']
                                && $this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['exist']
                                && !$this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['wrongstatus']['status']) {

                                $statusvalue = $this->orderstatus[$statusstring2]; //Get status number value

                                //Change item status
                                DB::table('cart')->where('voucher', $voucher['voucher_no'])
                                    ->update(['status' => $statusvalue]);
                                $check_status = false;
                                $report['pickup']['success'][] = (object)[
                                    'vouchers' => [$voucher],
                                    'status' => '200',
                                    'message' => 'Item status has been changed to ' . ucfirst($statusstring2)
                                ];
                            }
                            //Failed to proceed. Add to Errors array
                            if ($check_status) {
                                $message = '';
                                if (!$this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['action']) {
                                    $message = 'Item status is already set to ' . ucfirst($statusstring2);
                                } else if (!$this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['exist']) {
                                    $message = 'Item does not exist in the records';
                                } else if ($this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['wrongstatus']['status']) {
                                    $message = $this->proceedOrderAction($voucher['voucher_no'], $pick_status, $statusstring2)['wrongstatus']['message'];
                                }
                                $report['pickup']['errors'][] = (object)[
                                    'vouchers' => [$voucher['voucher_no']],
                                    'status' => '400',
                                    'message' => $message
                                ];
                            }
                            $check_status = true;
                        }

                        //End vouchers loop
                    }
//
                if(count($report['cancel']['success']) > 0 || count($report['pickup']['success']) > 0){

                    if(count($report['cancel']['success']) > 0){
                        $receipt_cancel = $this->buildSuccessDetails($report['cancel']['success']);

                    } else {

                        $receipt_cancel = array();
                    }

                    if(count($report['pickup']['success']) > 0){
                    $receipt_pickup = $this->buildSuccessDetails($report['pickup']['success']);
                    } else {

                        $receipt_pickup = array();
                    }


                    $order_data =  \App\Order::where('order_no', $report['order_no'])->select('user_id')->first();
                    $user_data =  \App\User::where('id', $order_data->user_id)->select('email', 'firstname', 'company_id', 'member_id')->first();
                    $token =  \App\Company::where('id', $user_data->company_id)->select('token')->first();

                    if(count($report['pickup']['success']) > 0 && count($report['cancel']['success']) > 0) {
                        $details = $this->buildTablePickupCancel($receipt_cancel, $receipt_pickup);
                        $send_status = $request->status;
                    } elseif ( count($report['pickup']['success']) > 0 && count($report['cancel']['success']) == 0){

                        $details = $this->buildTable($receipt_pickup,null, $report['order_no']);
                        $send_status = 8;
                    } else{
                        $details = $this->buildTable($receipt_cancel,null, $report['order_no']);
                        $send_status = 0;

                    }
                    
                    if(count($report['pickup']['success']) > 0) {
                        $branch_name = $receipt_pickup[0]['pickup_location_name'];
                    } else {
                        $branch_name = $receipt_cancel[0]['pickup_location_name'];
                    }


                    $filterd_vouchers = [];
                    foreach ($report['cancel']['success'] as $voucher) {
                        foreach ($voucher->vouchers as $voucher_2) {
                            array_push($filterd_vouchers, $voucher_2);
                        }
                    }

                    //Refund
                    try {

                        if(count($report['cancel']['success']) > 0) {

                            $refund = $this->refundToAccountWithoutDelivery($report['order_no'], $filterd_vouchers);

                            if($refund['status']) {

                                if($send_status == $request->status) {

                                    $email_payload = [
                                        'first_name' => $user_data->firstname,
                                        'email' => $user_data->email,
                                        //'details' => $details,
                                        'current_balance' => $refund['balance'],
                                        'refund_amount' => $refund['refund_amount'],
                                        'member_no' => $user_data->member_id,
                                        'order_no' => $report['order_no'],
                                        'date' => date("d-m-Y"),
                                        'details_pickup' => $details['pickup'],
                                        'details_cancel' => $details['cancel'],
                                        'branch_name' => $branch_name
                                    ];

                                }

                                if($send_status == 0) {
                                    $email_payload = [
                                        'first_name' => $user_data->firstname,
                                        'email' => $user_data->email,
                                        'details' => $details,
                                        'current_balance' => $refund['balance'],
                                        'refund_amount' => $refund['refund_amount'],
                                        'member_no' => $user_data->member_id,
                                        'order_no' => $report['order_no'],
                                        'item_name' => ''
                                    ];
                                }
                            }
                        } else {

                            $get_balance = $this->getUserBalance($order_data->user_id, $order_data->company_id);
                            if($send_status == $request->status) {
                                $email_payload = [
                                    'first_name' => $user_data->firstname,
                                    'email' => $user_data->email,
                                    //'details' => $details,
                                    'current_balance' => $get_balance['balance'],
                                    'refund_amount' => 0,
                                    'member_no' => $user_data->member_id,
                                    'order_no' => $report['order_no'],
                                    'date' => date("d-m-Y"),
                                    'details_pickup' => $details['pickup'],
                                    'details_cancel' => $details['cancel'],
                                    'branch_name' => $branch_name
                                ];

                            }

                            if($send_status == 8) {
                                $email_payload = [
                                    'first_name' => $user_data->firstname,
                                    'email' => $user_data->email,
                                    'details' => $details,
                                    'order_no' => $report['order_no'],
                                    'member_no' => $user_data->member_id,
                                    'branch_name' => $branch_name
                                ];
                            }
                        }

//dd($email_payload);
                            //Send Email
                            $email = $this->sendStatusChangeEmail($send_status, $token->token, $email_payload);
                            $report['email_log'][] = (object)[
                                'message' => $email,
                                'order_no' => $report['order_no'],
                                'status' => '200'
                            ];

                    }catch (\Exception $e) {
                        $report['refund_exception_errors'][] = (object)[
                            'message' => $e->getMessage(),
                            'error_trace' => $e->getTraceAsString(),
                            'order_no' => $report['order_no'],
                            'status' => '400'
                        ];
                    }

                }
            //}
            return response()->json(['data' => $report, 'status' => '200'], 200);

        }catch(\Exception $e){
            return response()->json(['data' => 'Error '.$e->getMessage(), 'status' => '400'], 400);
        }

    }
}